import ooeuler_getter

class P2WithRectangularFundamentalRegion(ooeuler_getter.Polyhedron):
    def __init__(self,opt):
        """
        Note that the union of
        { (i,j) | i=0,..,n_row-1,  j=0,..,n_col-1} and { (n_row,0) }
        is the complete system of representatives.
        """
        (n_row,n_col)=opt
        self.n_row=n_row
        self.n_col=n_col

        self.faces=[ (i,j) for i in range(n_row) for j in range(n_col) ]
        self.faces.append((n_row,0))

        rep={}
        for i in range(n_row+1):
            for j in range(n_col+1):
                rep[(i,j)] = self.get_equivalent_cell((i,j))

        self.edges=[]

        for i in range(n_row):
            for j in range(n_col):
                e=[rep[(i,j)], rep[(i,j+1)]]
                self.edges.append(frozenset(e))
                e=[rep[(i,j)], rep[(i+1,j)]]
                self.edges.append(frozenset(e))

        self.vertices=[]
        chartv=[]
        chartv.append((n_col+2)*[None])
        for i in range(n_row):
            cvi=[None]
            for j in range(n_col):
                v=[rep[(i,j)], rep[(i,j+1)], rep[(i+1,j)], rep[(i+1,j+1)]]
                self.vertices.append(frozenset(v))
                cvi.append(frozenset(v))
            cvi.append(None)
            chartv.append(cvi)
        chartv.append((n_col+2)*[None])

        self.face_as_edges={} 
        for f in self.faces:
            es = [ e for e in self.edges if f in e ]
            self.face_as_edges[f]=frozenset(es)

        self.edge_as_vertices={}
        for e in self.edges:
            vs = [ v for v in self.vertices if e.issubset(v)]
            self.edge_as_vertices[e]=frozenset(vs)

        self.edges=frozenset(self.edges)
        self.vertices=frozenset(self.vertices)

#        chartf=[[self.get_equivalent_cell((i,j)) for j in range(n_col+1)] for i in range(n_row+1)]
#        chart=(chartf,chartv)

#        self.layout_hint=[chart]
#        self.layout_hint_type="multisquare"


        chartf=[[self.get_equivalent_cell((i,j)) for j in range(n_col+1)] for i in range(n_row+1)]
        layout_hint_vedge=[[ True if fij or fik else False for (fij,fik) in zip(fi+[None],[None]+fi)] for fi in chartf]
        layout_hint_hedge=[[ True if fij else False for fij in chartf[0]]]+[[ True if fik or fjk else False for (fik,fjk) in zip(fi,fj)] for (fi,fj) in zip(chartf[1:],chartf)]+[[ True if fij else False for fij in chartf[-1]]]

        self.layout_hint=(chartf,layout_hint_hedge,layout_hint_vedge,chartv)
        self.layout_hint_type="xsquare"

    def get_equivalent_cell(self,cell):
        (i,j)=cell
        if i<self.n_row:
            if j<self.n_col:
                return (i,j)
            else:
                return (self.n_row-i,0)
        else:
            if j==0:
                return (i,0)
            else:
                return (0,self.n_col-j)

